"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TemplateReader = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const _config_1 = require("./_config");
const promises_1 = require("fs/promises");
class TemplateReader {
    constructor(templateName, templateDirInfo, templateFilesInfo, templateConfigFile) {
        this.templateName = templateName;
        this.templateDirInfo = templateDirInfo;
        this.templateFilesInfo = templateFilesInfo;
        this.templateConfigFile = templateConfigFile;
    }
    static build(wtedTemplateName) {
        return __awaiter(this, void 0, void 0, function* () {
            const allTemplatesDirInfoResult = yield this.getAllTemplateDirInfo();
            if (allTemplatesDirInfoResult instanceof Error)
                return allTemplatesDirInfoResult;
            let templateName;
            const templateDirInfoResult = this.findWantedTemplate(wtedTemplateName, allTemplatesDirInfoResult);
            if (templateDirInfoResult instanceof Error)
                return templateDirInfoResult;
            else
                templateName = wtedTemplateName;
            const templateConfigFile = this.getTemplateConfigFile(templateDirInfoResult);
            const templateFilesInfosResult = yield this.getAllTemplateFilesInfo(templateDirInfoResult.dirPath);
            if (templateFilesInfosResult instanceof Error)
                return templateFilesInfosResult;
            return {
                templateName,
                templateDirInfo: templateDirInfoResult,
                templateFilesInfo: templateFilesInfosResult,
                templateConfigFile
            };
        });
    }
    static findWantedTemplate(wantedTemplateName, templatesDirsInfo) {
        const templateDirFound = templatesDirsInfo.find(tempDirInfo => this.getTemplateName(tempDirInfo.dirName) === wantedTemplateName);
        if (!templateDirFound)
            return new Error(`Aucune template nommé "${wantedTemplateName}" n'as été trouvé`);
        else
            return templateDirFound;
    }
    static getAllTemplateDirInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield (0, promises_1.readdir)(_config_1.absolutePathConfigDir, { withFileTypes: true });
                const dirInfo = result
                    .filter(dirent => dirent.isDirectory())
                    .map(dirent => ({
                    dirName: dirent.name,
                    dirPath: `${_config_1.absolutePathConfigDir}/${dirent.name}`
                }));
                const checkingResult = this.checkAllTemplateDirInfo(dirInfo);
                return checkingResult instanceof Error ? checkingResult : dirInfo;
            }
            catch (err) {
                return new Error(`Une erreur innatendue s'est produite : ${err}`);
            }
        });
    }
    static getAllTemplateFilesInfo(templateDirPath) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield (0, promises_1.readdir)(templateDirPath, { withFileTypes: true });
                return result
                    .filter(file => file.isFile() && file.name !== _config_1.nameComponentConfigFile)
                    .map(file => ({
                    fileName: file.name,
                    filePath: `${templateDirPath}/${file.name}`,
                }));
            }
            catch (err) {
                return new Error(`Une erreur innatendue s'est produite : ${err}`);
            }
        });
    }
    static checkAllTemplateDirInfo(templatesDirInfos) {
        const errors = [];
        for (const templateDirInfos of templatesDirInfos) {
            const templateDirName = templateDirInfos.dirName;
            const templateDirNameParted = _config_1.regexDirectory.exec(templateDirName);
            if (!templateDirNameParted || !templateDirNameParted[1])
                errors.push(`Problème sur le template ${templateDirName} :\n ${_config_1.ERR_TEMPLATE_DIR_NAME_PART1}`);
            else if (!templateDirNameParted || !templateDirNameParted[2])
                errors.push(`Problème sur le template ${templateDirName} : ${_config_1.ERR_TEMPLATE_DIR_NAME_PART2}`);
        }
        if (errors.length) {
            const textError = errors.join(`\n-----------------------------`);
            return new Error(textError);
        }
    }
    static getTemplateName(templateDirName) {
        const templateDirNameParted = _config_1.regexDirectory.exec(templateDirName);
        return templateDirNameParted[1];
    }
    static getTemplateConfigFile(templateDirInfos) {
        const templateDirPath = templateDirInfos.dirPath;
        if (fs_1.default.existsSync(`${templateDirPath}/${_config_1.nameComponentConfigFile}`)) {
            const result = fs_1.default.readFileSync(path_1.default.resolve(`${templateDirPath}/${_config_1.nameComponentConfigFile}`));
            return JSON.parse(result.toString());
        }
        else
            console.log(`Infos : Aucun fichier de configuration n'as été fourni avec le template : ${templateDirInfos.dirName}`);
        return null;
    }
}
exports.TemplateReader = TemplateReader;
//# sourceMappingURL=TemplateReader.js.map